-- Cooldown Module

local width, height = 250, 22

local cdtimers = LibStub("LibCandyBar-3.0")

function SN:CoolDowns()

	SN.F.cooldowns = CreateFrame("Frame", "CDFrame", UIParent)
	cds = SN.F.cooldowns
	cds.bars = {}
	cds:SetPoint("CENTER", UIParent, "CENTER", 0, -200)
	cds:SetSize(width + 8, height + 8)
	cds:SetBackdrop({ 
				bgFile = SN.V.media.blankTex,
				edgeFile = SN.V.media.blankTex,
				tile = true,
				tileSize = 4,
				edgeSize = 4, 
				insets = { left = 4, right = 4, top = 4, bottom = 4 }
	})
	cds:SetBackdropColor(0.1, 0.1, 0.1, 0.4)
	cds:SetBackdropBorderColor(0.1, 0.1, 0.1, 1)
	cds:SetScript("OnUpdate", function(self)
				if #self.bars > 0 then 
					cds:SetSize(width + 8, (height * #self.bars) + 8)
				else
					cds:SetSize(width + 8, height + 8)
				end
			end)
end

function SN:CreateCDBar(caster, spellid, icon, cdtimer, index)
	
	--Start the CD Timer
	local bar = cdtimers:New(SN.V.media.minTex, width, height)
	cds.bars[bar] = true
	local class, classFileName = UnitClass(caster)
	local color = { r = 0.0, g = 0.44, b = 0.87 } --RAID_CLASS_COLORS[classFileName]
	bar:SetIcon(icon)
	bar:Set("SN:anchor", cds)
	bar:Set("SN:key", caster)
	bar:SetParent(cds)
	bar:SetFrameStrata("MEDIUM")
	bar:SetColor( 0.0, 0.44, 0.87);	
	bar:SetDuration(cdtimer)
	bar:SetClampedToScreen(true)
	local unit = strsplit("-",caster)
	local cname, crealm = UnitName(caster)
	bar:SetLabel(cname)
	bar:SetPoint("TOPLEFT",cds,"TOPLEFT", 4, -4)
	bar.candyBarLabel:SetJustifyH("LEFT")
	
	bar:Start()
end

function SN:StartCD(frame,cooldown,text,guid,caster,frameicon, spell)
	if not (SN.curr[cooldown['spellID']][guid]) then
	   SN.curr[cooldown['spellID']][guid]=guid
   end
	
	if(SN.profileDB.castannounce) then
		local name = select(1, GetSpellInfo(cooldown['spellID']))
		local cd = cooldown['CD'] / 60
		if(RI:GroupType()==2) then
			SendChatMessage(caster.." Casts "..name.." "..cd.."Min".." CD" ,"RAID");
		else
			SendChatMessage(caster.." Casts "..name.." "..cd.."Min".." CD" ,"PARTY");
		end
	end
		
	local bar = SN:CreateBar(frame,cooldown,caster,frameicon,guid)
	
	local args = {cooldown,guid,frame,text,bar,caster,spell}
	local handle = BLRCD:ScheduleTimer("StopCD", cooldown['CD'],args)
	SN['handles'][guid] = BLRCD['handles'][guid] or {}
	BLRCD['handles'][guid][spell] = {args,handle,bar}
end


function SN:StopCD(args)
	SN.curr[args[1]['spellID']][args[2]] = nil;
	
	local a = args[5]:Get("cds:anchor")
	if a and a.bars and a.bars[args[5]] then
      a.bars[args[5]] = nil
		SN:RearrangeBars(a) 
	end
	
	if(SN.profileDB.cdannounce) then
		local name = select(1, GetSpellInfo(args[1]['spellID']))
		if(RI:GroupType()==2) then
			SendChatMessage(args[6].."'s "..name.." CD UP" ,"RAID");
		else
			SendChatMessage(args[6].."'s "..name.." CD UP" ,"PARTY");
		end
	end
		
	args[4]:SetText(SN:GetTotalCooldown(args[1]))
end

function BLRCD:UpdateCooldown(frame,event,unit,cooldown,text,frameicon, ...)
	if(event == "COMBAT_LOG_EVENT_UNFILTERED") then
		local timestamp, type,_, sourceGUID, sourceName,_,_, destGUID, destName = select(1, ...)
		if(type == cooldown['succ']) then
			local spellId, spellName, spellSchool = select(12, ...)
			if(spellId == cooldown['spellID']) then
				if (LibRaidInspectMembers[sourceGUID]) then
					--0BLRCD:CheckSpecial(sourceGUID,spellName)
					BLRCD:StartCD(frame,cooldown,text,sourceGUID,sourceName,frameicon, spellName)
					text:SetText(BLRCD:GetTotalCooldown(cooldown))
	         end
			end
		 end
	elseif(event =="GROUP_ROSTER_UPDATE") then
		if not(RI:GroupType() == 2 or RI:GroupType() == 1) then
			BLRCD:UpdateRoster(cooldown)
	      BLRCD:CancelBars(frameicon)
		end
		text:SetText(BLRCD:GetTotalCooldown(cooldown))
	elseif(event =="LibRaidInspect_Remove") then
		local guid, name = select(1, ...)
		BLRCD:RemovePlayer(guid)
	else
		text:SetText(BLRCD:GetTotalCooldown(cooldown))
	end	
end

function BLRCD:GetTotalCooldown(cooldown)
	local cd = 0
	local cdTotal = 0
	for i,v in pairs(BLRCD.cooldownRoster[cooldown['spellID']]) do
		cdTotal=cdTotal+1
	end
	
	for i,v in pairs(BLRCD.curr[cooldown['spellID']]) do
		cd=cd+1
	end
	
	local total = (cdTotal-cd)
	if(total < 0) then
		total = 0
	end
		
	return total
end

local function barSorter(a, b)
	return a.remaining < b.remaining and true or false
end

function SN:RearrangeBars(anchor)
	if not anchor then return end
    if not next(anchor.bars) then return end
    local frame = anchor:GetParent()
    wipe(SN.tmp)
	
    for bar in pairs(anchor.bars) do
		SN.tmp[#SN.tmp + 1] = bar
	end
	
	if(#SN.tmp>2)then
		frame:SetHeight(14*#SN.tmp);
	else
		frame:SetHeight(18);
	end

	table.sort(SN.tmp, barSorter)
	local lastDownBar, lastUpBar = nil, nil
	
	for i, bar in next, SN.tmp do
		local spacing = -6
		bar:ClearAllPoints()
		if not (lastDownBar) then
			bar:SetPoint("TOPLEFT",anchor,"TOPLEFT", 4, -4)
		else    
    		bar:SetPoint("TOPLEFT", lastDownBar, "TOPLEFT", 4, -4)
		end
		lastDownBar = bar
	end
end